libname RDExamp "C:\users\sasdemo\RD_Examples";
libname HPExpLib "C:\users\sasdemo\RD_Examples";
%let test_env = hp_marketrisk;
%let cube_dir = C:\users\sasdemo\RD_Examples;

/* Before you run the HPRISK procedure in distributed mode, you might need to specify the 
   following options. Contact your administrator about whether to use them and the 
   appropriate values to specify if you do.

GRIDHOST       - the host name for the grid.  
GRIDINSTALLLOC - the path to the SAS High-Performance Risk installation on the grid.
GRIDRSHCOMMAND - the command to run a remote shell.
GRIDREPLYHOST  - the host name of the client node to which the grid connects. 
                 Specify a fully qualified domain name (for example, rdgrd001.orion.com). 
                 Use this option when the client node has more than one network controller, 
                 or when you need to specify a full network name.
GRIDPORTRANGE  - the ports that a firewall permits. Specify this option when the 
                 client-to-grid connection goes through a firewall.

To run HPRISK in SMP mode, set GRIDHOST and GRIDINSALLLOC to empty.  
Do not set the other options. Set the FILEPREFIX option to the local folder.
*/

/* Create a new SAS Risk Dimensions Environment */

option set=GRIDHOST="";
option set=GRIDINSTALLLOC="";  

proc risk;
  env new=RDExamp.&test_env;
  env save;
run;


/* Generate a Portfolio */

%let ninst = 10000;
data RDExamp.instdata;
		length insttype $12 instid $12 region $8 trader_id $16 holding 5;
		do i = 1 to &ninst;
			if ranuni(54321) < 0.5 then insttype = "TechCompany";
		else insttype = "BankCompany";

		region_rand = rand('table',.15, .10, .35, .40);
			select (region_rand);
			when (1) do;Region = "North";
				trader_rand = 100 + rand('table',.30,.50,.20);
				Trader_ID = cats("Trader_",trader_rand);
				end;
			when (2) do; Region = "West";
				trader_rand = 200 + rand('table',.30,.50,.20);
				Trader_ID = cats("Trader_",trader_rand);
				end;
			when (3) do; Region = "South";
				trader_rand = 300 + rand('table',.30,.50,.20);
				Trader_ID = cats("Trader_",trader_rand);
	        	end;
			when (4) do; Region = "East";
				trader_rand = 400 + rand('table',.30,.50,.20);
				Trader_ID = cats("Trader_",trader_rand);
				end;
			otherwise;
			end;

			instid = cats("instid_",i); 
			holding = ceil(ranuni(54321)*100);
			output;
			end;
			drop i region_rand trader_rand;
run;

/* Create Instrument Data Set */

proc risk;
  env open=RDExamp.&test_env;
  instdata Instruments file=RDExamp.instdata format=simple;
  declare instvars = (Trader_ID char 16 class,
  					 		  Region char 8 class)
  ;
  env save;
run;

/* Create and Register Market Data */

data RDExamp.mktdata;
  TEC=15; 
  BNK=30;  
;

data RDExamp.covdata;
	length _name_ _type_ $8;
	_type_ = "COV";
	input _name_ $ TEC BNK;  
	datalines;
TEC      .0004   .00006
BNK      .00006   .0001
;

proc risk;
	env open=RDExamp.&test_env;
	declare riskfactors=(TEC num var, BNK num var);
	marketdata Market file=RDExamp.mktdata  type=current;
	marketdata Cov    file=RDExamp.covdata  type=covariance;
	env save;
run;

/* Define Valuation Methods */

proc compile env=RDExamp.&test_env outlib=RDExamp.&test_env package = distortion;
  method Tech_Price kind=price;
    _value_ = TEC;
  endmethod;
  method Bank_Price kind=price;
    _value_ = BNK;
  endmethod;

  method myVaR kind= computed ;
    Value_VaR = get_queryval('_value_', 'VaR');
    myVaR = Value_VaR * 1.05;
  endmethod;


function myMean(dist[*]);
  n = dim(dist);
  total = 0;
  do i = 1 to n;
    total = total + dist{i};
  end;
  total = 2*(total/n);
  return (total);
endsub;

/* Specify the Analysis and Create the Project */

proc risk;
  	env open=RDExamp.&test_env;
  	instrument TechCompany methods=(Price Tech_Price) 
              variables = (holding Trader_ID Region);
  	instrument BankCompany methods=(Price Bank_Price) 
              variables = (holding Trader_ID Region);
  	env save;
run;

proc risk;
  env open=RDExamp.&test_env;
  sources InstDataSource (Instruments);
  read sources=InstDataSource out=Portfolio;
  crossclass MarketRiskCC (Insttype Trader_ID Region);
  simulation Covariance method=covariance
          ndraws=100
          outvars=(pl)
          data=Cov
		  interval = weekday;
  project Project
    	  analysis=(Covariance)
          computedmethods=(myVaR)
          portfolio=Portfolio
          crossclass=MarketRiskCC
          data=( Market );
  env save;
run;

/* Export the Project */

proc hpexport
   env = RDExamp.&test_env
   project = Project
   expprojlib = HPExpLib
   ;
   userstat myMean func=myMean dist_type=standard; 
run;

/* Run the HPRISK Procedure.
   You can only open the cube that is created on a system running
   SAS High-Performance Risk in SMP mode.
*/

proc hprisk
	expprojlib = HPExpLib
	cube = "&cube_dir\HPRisk_Example"
	filesprefix = "&cube_dir\HPRisk_Example"
	task = RUNPROJECT
	;
	crossclassvars 
		Region
		InstType 	
		Trader_ID  
		InstID
		;
	performance nodes = 1 nthreads = 1;
quit;


/* Query the cube */

proc hprisk
	cube = "&cube_dir\HPRisk_Example"
	task = QUERY
	;
	output
		summary = RDexamp.hp_summary
	;
	query 
		Region
	;
quit;


/* When you try to create a risk cube and the specified risk cube 
   already exists, the run fails.  You must clean the risk cube 
   in order to prevent this from happening.


proc hprisk
	task = CLEAN
	cube = "&cube_dir\HPRisk_Example"
	;
quit; 
*/